"""
Users
"""
from json import dumps
from . import utilities as util
from . import _constants as const
from . import exceptions as e

class User(util.Resource):
    """
    Create and manage users in a Sonetel account
    """

    def __init__(self, access_token: str):
        if not access_token:
            raise e.AuthException('access_token is required')

        super().__init__(access_token=access_token)
        self._url = f'{const.API_URI_BASE}{const.API_ENDPOINT_ACCOUNT}{self._accountid}{const.API_ENDPOINT_USER}'

    def get(self, all_users: bool = False, userid: str = ''):
        """
        Fetch details about all users or a specific user.

        If userid is not included with the request, details of the current user are fetched.

        :param all_users: Boolean. Optional. Get a list of all the users in the account. Defaults to False.
        :param userid: String. Optional. ID of a specific user to get the information for.
        """

        url = self._url

        if userid:
            url += userid
        elif not all_users:
            url += self._userid

        return util.send_api_request(
            token=self._token,
            uri=url,
            method='get') if util.is_valid_token(self._decoded_token) else False

    def add(self,
            email: str,
            f_name: str,
            l_name: str,
            password: str,
            user_type: str = 'regular'
            ) -> dict:
        """
        Adds a new user. Account admin privilege required.

        :param email: Required. String. The email address of the user.
        :param f_name: Required. String. The first name of the user
        :param l_name: Required. String. The last name of the user.
        :param password: Required. String. The password to be used.
        :param user_type: Required. String. The privilege level of the new user. Accepted values regular and admin.
        Defaults to regular.
        """

        # Checks
        if not password:
            return util.prepare_error(
                code=const.ERR_USER_DETAIL_EMPTY,
                message='password cannot be empty'
            )

        if not email:
            return util.prepare_error(
                code=const.ERR_USER_DETAIL_EMPTY,
                message='email cannot be empty'
            )

        if not f_name:
            return util.prepare_error(
                code=const.ERR_USER_DETAIL_EMPTY,
                message='first name cannot be empty'
            )

        if not l_name:
            return util.prepare_error(
                code=const.ERR_USER_DETAIL_EMPTY,
                message='last name cannot be empty'
            )

        # Request
        url = self._url
        body = {
            "user_fname": f_name,
            "user_lname": l_name,
            "email": email,
            "password": password,
            "type": user_type
        }

        return util.send_api_request(
            token=self._token,
            uri=url,
            method='post',
            body=dumps(body)
        )

    def delete(self, userid: str):
        """
        Delete a user from your Sonetel account.

        :param userid: String. Required. The unique ID of the user to be deleted.
        """
        if not userid:
            return util.prepare_error(
                code=const.ERR_USED_ID_EMPTY,
                message='user id cannot be empty'
            )

        url = self._url + userid

        return util.send_api_request(
            token=self._token,
            uri=url,
            method='delete'
            )

    def update(self, request: dict):
        """
        update user settings
        """
        raise NotImplementedError



import os
from sonetel import Auth
from sonetel import Recording
from sonetel import exceptions as e

def access_token():
    try:
        auth = Auth(username=os.getenv('SonetelUsername'), password=os.getenv('SonetelPassword'))
        return auth.get_access_token()
    except e.AuthException as error:
        raise e.AuthException(f"Error: {error}")


token = access_token()

def test_recording_get_all():
    recording = Recording(access_token=token)
    result = recording.get()

    assert result['status'] == 'success'
    assert type(result['response']) == list

def test_recording_get_single():
    recording = Recording(access_token=token)
    result = recording.get(rec_id='REd2jiyro9azqj')

    assert result['status'] == 'success'
    assert type(result['response']) == dict
    assert result['response']['call_recording_id'] == 'REd2jiyro9azqj'
    assert result['response']['account_id'] == 5033728

def test_recording_get_single_invalid_id():
    recording = Recording(access_token=token)
    result = recording.get(rec_id='invalid_id')

    assert result['status'] == 'failed'
    assert result['response']['detail'] == 'Invalid recording id'



"""
Sonetel
"""

from .account import Account
from .auth import Auth
from .calls import Call
from .phonenumber import PhoneNumber
from .recording import Recording
from .users import User
from .voiceapps import VoiceApp



"""
Utilities for internal use
"""
from time import time
import datetime
import jwt
import requests
from . import _constants as const
from . import exceptions as e

class Resource:
    """
    Base recource class for Sonetel API
    """
    def __init__(self, access_token: str):

        if not access_token:
            raise e.AuthException("access_token is a required parameter.")
        self._token: str = access_token
        self._decoded_token = decode_token(self._token)
        self._accountid: str = self._decoded_token['acc_id']
        self._userid: str = self._decoded_token['user_id']

        if not is_valid_token(self._decoded_token):
            raise e.AuthException("Token has expired")

# Static methods

def is_valid_token(decoded_token: dict) -> bool:
    """
    Return True if token hasn't expired. Accepts a decoded token.
    """
    # TODO: If token has expired, try to refresh it
    return decoded_token['exp'] - int(time()) > 60

def is_valid_date(date_text):
    """
    Check if the passed date is in the correct format
    """
    # Based on https://stackoverflow.com/a/16870699/18276605
    try:
        datetime.datetime.strptime(date_text, '%Y%m%dT%H:%M:%SZ')
        return True
    except Exception:
        return False

def date_diff(start, end):
    """
    Check if the end date is greater than start date. Returns a boolean.
    """
    start_date = datetime.datetime.strptime(start, '%Y%m%dT%H:%M:%SZ').strftime('%s')
    end_date = datetime.datetime.strptime(end, '%Y%m%dT%H:%M:%SZ').strftime('%s')
    return int(end_date) - int(start_date) > 0

def decode_token(token) -> dict:
    """
    Decode the JWT token
    """
    return jwt.decode(
        token,
        audience='api.sonetel.com',
        options={"verify_signature": False}
    )


def send_api_request(token: str,
                     uri: str,
                     method: str = 'GET',
                     body: str = None,
                     body_type: str = const.CONTENT_TYPE_GENERAL) -> dict:
    """
    Send an API request to Sonetel.

    :param token: Required. String. The access token.
    :param uri: Required. String. The API endpoint to send the request to.
    :param method: Optional. String. The HTTP method to use. Defaults to GET.
    :param body: Optional. String. The body of the request. Defaults to None.
    :param body_type: Optional. String. The content type of the body. Defaults to application/json.
    :return: A dictionary containing the response.
    """

    # Checks
    if not token:
        raise e.SonetelException('"token" is a required parameter')
    if not uri:
        raise e.SonetelException('"uri" is a required parameter')

    # Prepare the request Header
    request_header = {
        "Authorization": "Bearer " + token,
        "Content-Type": body_type,
        "User-Agent": f'Sonetel Python Package - v{const.PKG_VERSION}'
    }

    # Send the request
    try:
        r = requests.request(
            method=method,
            url=uri,
            headers=request_header,
            data=body,
            timeout=60
        )
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        return {'status': 'failed', 'error': 'HTTPError', 'message': err.response.text}
    except requests.exceptions.ConnectionError as err:
        return {'status': 'failed', 'error': 'ConnectionError', 'message': err}
    except requests.exceptions.RequestException as err:
        return {'status': 'failed', 'error': 'RequestException', 'message': err}

    # pylint: disable=no-member
    if r.status_code == requests.codes.ok:
        return r.json()

    r.raise_for_status()

def prepare_error(code: int, message: str) -> dict:
    """
    Prepare a dict with the error response.
    """
    return {
        'status': 'failed',
        'code': code,
        'message': message
    }



"""
Constants: API urls and other static data
"""

# General
API_URI_BASE = 'https://public-api.sonetel.com'
API_URI_AUTH = 'https://api.sonetel.com/SonetelAuth/beta/oauth/token'
API_AUDIENCE = 'api.sonetel.com'
PKG_VERSION = '0.2.1'

# Headers
CONTENT_TYPE_GENERAL = 'application/json;charset=UTF-8'
CONTENT_TYPE_AUTH = 'application/x-www-form-urlencoded'

# Authentication
CONST_JWT_USER = 'sonetel-api'
CONST_JWT_PASS = 'sonetel-api'
CONST_TYPES_GRANT = ['password', 'refresh_token']
CONST_TYPES_REFRESH = ['yes', 'no']

# API Resources
API_ENDPOINT_ACCOUNT = '/account/'
API_ENDPOINT_CALLBACK = '/make-calls/call/call-back'
API_ENDPOINT_NUMBERSUBSCRIPTION = '/phonenumbersubscription/'
API_ENDPOINT_VOICEAPP = '/voiceapp/'
API_ENDPOINT_USER = '/user/'
API_ENDPOINT_CALL_RECORDING = '/call-recording'

# Users
CONST_TYPES_USER = ['regular', 'admin']

# Phone Numbers
CONST_CONNECT_TO_TYPES = ['user', 'phnum', 'sip', 'app']

# Error codes
ERR_NUM_NOT_E164 = 1000
ERR_NUM_UPDATE_EMPTY = 1001
ERR_USER_DETAIL_EMPTY = 2000
ERR_USED_ID_EMPTY = 2001
ERR_ACCOUNT_UPDATE_BODY_EMPTY = 3000
ERR_CALLBACK_NUM_EMPTY = 4000




import os
from uuid import uuid4

import pytest

from sonetel import VoiceApp
from sonetel import exceptions as e
from tests.common_functions import access_token

token = access_token()
if token:
    voice_app = VoiceApp(access_token=token)
else:
    raise e.AuthException("Cannot get access token")


def test_voice_app_init():
    # Test if the voice app object is initialized
    assert voice_app is not None


def test_voice_app_get():
    # Get all voice apps
    response = voice_app.get()
    assert response["status"] == "success"



"""
Make phone calls using Sonetel's CallBack API.
"""
# Import Packages.
from json import dumps
from . import utilities as util
from . import _constants as const
from . import exceptions as e

class Call(util.Resource):
    """Use Sonetel's CallBack API to make business quality international calls at the cost of 2 local calls.

    See the [API documentation](https://docs.sonetel.com/docs/sonetel-documentation/YXBpOjE1OTMzOTIy-make-calls) for more information.

    Examples:
        >>> call = Call(access_token='your_access_token')
        >>> call.callback(num1='+12125551234', num2='+44123456789')
        {'statusCode': 202, "response": {"session_id": "1234567890"}}
    
    Args:
        access_token (str, required): The access token generated from the Auth class.
        app_name (str, optional): The name of the app that is making the request. Defaults to 'PythonSonetelPackage'. This is used to identify the app in logs.
    
    Returns:
        dict (JSON): Return the status code and message as a dict.
    """

    def __init__(self, access_token, app_name: str = 'PythonSonetelPackage') -> None:
        if not access_token:
            raise e.AuthException('access_token is required')

        super().__init__(access_token=access_token)
        self._url = f'{const.API_URI_BASE}{const.API_ENDPOINT_CALLBACK}'

        if len(app_name) < 2:
            raise e.SonetelException('app_name must be at least 2 characters long')
        
        self._app_name = f"{app_name}__{self._accountid}__{self._userid}"

    def callback(self, num1: str, num2: str, cli1: str = 'automatic', cli2: str = 'automatic'):
        """

        ## Number Format

        It is recommended that both the phone numbers (num1 and num2) be entered in the international E164 format with a
        leading +. For example, if you want to call a US number (212) 555-1234, it should be set as `+12125551234`.

        However you can also provide SIP addresses. Additionally, `num1` can be your Sonetel username - this will make
        sure that the incoming call to you is handled as per your incoming settings defined in the app.

        ## Caller ID
        
        It is best to use 'automatic' CLI as our system selects the best possible phone to be shown from the numbers
        available in your account. If you don't have a Sonetel number, then your verified mobile number is used as CLI.
        Read more at https://sonetel.com/cli

        Args:
            num1 (str): The first phone number that will be called. This should be your phone number, SIP address or Sonetel email address.
            num2 (str): The phone number or address that you wish to connect to.
            cli1 (str): Optional. The caller ID shown to the first person. Defaults to `automatic`.
            cli2 (str): Optional. The caller ID shown to the second person. Defaults to `automatic`.

        Returns:
            response (dict): Return the JSON response from the API as a dict.
        """

        # Check if num1 and num2 are defined.
        if num1 and num2:

            # Initiate the callback
            body = {
                "app_id": f'{self._app_name}-{const.PKG_VERSION}',
                "call1": num1,
                "call2": num2,
                "show_1": cli1,
                "show_2": cli2
            }

            return util.send_api_request(
                token=self._token,
                uri=self._url,
                method='post',
                body=dumps(body)
            )
        return util.prepare_error(
            code=const.ERR_CALLBACK_NUM_EMPTY,
            message='num1 & num2 are required to make a call.'
        )



"""
Manage call recordings
"""
# Import Packages.
from . import utilities as util
from . import _constants as const

class Recording(util.Resource):
    """
    Class representing the call recording resource.
    """

    def __init__(self, access_token: str = None):
        super().__init__(access_token)
        self._url = f'{const.API_URI_BASE}{const.API_ENDPOINT_CALL_RECORDING}'

    def get(self,
            start_time: str = None,
            end_time: str = None,
            file_access_details: bool = False,
            voice_call_details: bool = False,
            rec_id: str = None
            ):
        """
        Get a list of all the call recordings or a single recording.
        Add the start_time and end_time to filter the recordings based on the created date.
        When a recording ID is provided, the start_time and end_time are ignored.

        To get the details needed to download the recording file, set file_access_details to True.

        :param start_time: The start timestamp in the format YYYYMMDDTHH:MM:SSZ. Example 20201231T23:59:59. Limit the results to recordings created after this timestamp. Not used when a recording ID is provided.
        :param end_time: The end timestamp in the format YYYYMMDDTHH:MM:SSZ. Example 20221123T18:59:59. Limit the results to recordings created before this timestamp. Not used when a recording ID is provided.
        :param rec_id: The unique recording ID. If not included, returns all the recordings.
        :param file_access_details: Boolean. Include the details needed to download recordings.
        :param voice_call_details: Boolean. Include the details of the voice calls.
        """

        url = self._url
        field_prefix = '&'

        # Prepare the request URL based on the params passed to the method
        if rec_id:
            # Get a single recording
            url += f'/{rec_id}'
            field_prefix = '?'
        else:
            # Search for and return multiple recordings
            url += f'?account_id={self._accountid}'

            if util.is_valid_date(start_time) and util.is_valid_date(end_time) and util.date_diff(start_time, end_time):
                url += f'&created_date_max={end_time}&created_date_min={start_time}'

        fields = []

        if file_access_details:
            fields.append('file_access_details')
        if voice_call_details:
            fields.append('voice_call_details')

        if len(fields) > 0:
            url += f'{field_prefix}fields=' + ','.join(fields)

        return util.send_api_request(token=self._token, uri=url, method='get')

    def delete(self, rec_id: str) -> dict:
        """
        Delete a call recording.

        :param rec_id: The ID of the recording that should be deleted
        :returns: A representation of the deleted recording.
        """
        url = f'{self._url}/{rec_id}'
        return util.send_api_request(token=self._token, uri=url, method='delete')



"""
Run tests on the Auth class
"""
import os
import sonetel as sntl

auth = sntl.Auth(username=os.getenv('SonetelUsername'), password=os.getenv('SonetelPassword'))

def test_get_access_token():
    access_token = auth.get_access_token()
    assert access_token is not None
    assert isinstance(access_token, str)
    assert access_token[:15] == 'eyJhbGciOiJSUzI'

def test_get_refresh_token():
    refresh_token = auth.get_refresh_token()
    assert refresh_token is not None
    assert isinstance(refresh_token, str)
    assert refresh_token[:15] == 'eyJhbGciOiJSUzI'

def test_get_decoded_token():
    decoded_token = auth.get_decoded_token()
    assert decoded_token is not None
    assert isinstance(decoded_token, dict)
    assert decoded_token['iss'] == 'SonetelNode123'
    assert decoded_token['aud'] == 'api.sonetel.com'
    assert 'user.read' in decoded_token['scope']
    assert 'user.write' in decoded_token['scope']

def test_refresh_token():
    refresh_token = auth.get_refresh_token()
    auth.create_token(refresh='yes', grant_type='refresh_token', refresh_token=refresh_token)
    access_token = auth.get_access_token()
    refresh_token = auth.get_refresh_token()
    assert auth.get_access_token() is not None
    assert auth.get_refresh_token() is not None
    assert auth.get_decoded_token() is not None
    assert isinstance(access_token, str)
    assert access_token[:15] == 'eyJhbGciOiJSUzI'
    assert refresh_token is not None
    assert isinstance(refresh_token, str)
    assert refresh_token[:15] == 'eyJhbGciOiJSUzI'



"""
Add and manage phone numbers in your Sonetel account.
"""
import re
from json import dumps
from . import utilities as util
from . import _constants as const
from . import exceptions as e


def is_e164(number: str) -> bool:
    """
    Checks if a number is e164 formatted. Returns True if it is, False if it isn't.

    :param number: The number to check. For example, +441234567890.
    :return: True if the number is e164 formatted, False if it isn't.
    """
    if re.search(r'^\+?[1-9]\d{7,15}$', number):
        return True
    return False


class PhoneNumber(util.Resource):
    """
    Phone number class
    """

    def __init__(self, access_token):
        if not access_token:
            raise e.AuthException('access_token is required')

        super().__init__(access_token=access_token)
        self._url = f'{const.API_URI_BASE}{const.API_ENDPOINT_ACCOUNT}{self._accountid}' \
                    f'{const.API_ENDPOINT_NUMBERSUBSCRIPTION}'

    def get(self, e164only: bool = True, number: str = '') -> dict:
        """
        List all the phone numbers present in the account.

        :param e164only: Optional. Boolean. Only return a list of phone numbers if set to True.
        Set to True by default.
        :param number: Optional. String. If you only want information about one of your numbers, pass it as a string.

        **DOCS**: https://docs.sonetel.com/docs/sonetel-documentation/YXBpOjE2MjQ3MzI4-phone-numbers

        :return: Information about the numbers assigned to account.
        """

        url = self._url

        if not isinstance(number, str):
            number = str(number)

        if number:
            if is_e164(number):
                url += number
            else:
                return util.prepare_error(
                    code=const.ERR_NUM_NOT_E164,
                    message=f'"{number}" is not a valid e164 number'
                )

        api_response = util.send_api_request(token=self._token, uri=url)
        response = api_response['response']

        # No numbers are found
        if response == 'No entries found':
            return {
                'status': 'success',
                'response': 'No entries found'
            }

        # Only return a list of e164 numbers, without any additional metadata
        if e164only:
            nums = []
            for entry in response:
                nums.append(entry['phnum'])
            return {
                'status': 'success',
                'response': nums
            }

        # Return full response
        return {
                'status': 'success',
                'response': response
            }

    def add(self, number: str) -> dict:
        """
        Buy a phone number that is available. Numbers that are available for purchase can be checked
        from the ``/availablephonenumber`` API endpoint.

        **DOCS**: https://docs.sonetel.com/docs/sonetel-documentation/YXBpOjE2MjQ3MzI4-phone-numbers

        :param number: the phone number you want to purchase.
        :return: Dict containing the success response or an error message.
        """
        if not isinstance(number, str):
            number = str(number)

        # Request body
        if is_e164(number):
            body = {
                "phnum": number
            }
        else:
            return util.prepare_error(
                code=const.ERR_NUM_NOT_E164,
                message=f'"{number}" is not a valid e164 number'
            )

        return util.send_api_request(
            token=self._token,
            uri=self._url,
            method='post',
            body=dumps(body)
        )

    def delete(self, number: str):
        """
        Remove a number from account. The phone number is removed immediately and cannot be recovered.

        :param number: The phone number to remove from account.
        :return: Dict containing the success response or an error message.
        """
        if not isinstance(number, str):
            number = str(number)

        if is_e164(number):
            url = f'{self._url}{number}'
        else:
            return util.prepare_error(
                code=const.ERR_NUM_NOT_E164,
                message=f'"{number}" is not a valid e164 number'
            )

        return util.send_api_request(
            token=self._token,
            uri=url,
            method='delete'
        )

    def update(self, number: str, connect_to_type: str, connect_to) -> dict:
        """
        Update the number's call forwarding settings.

        :param number: E164number for which the settings should be updated
        :param connect_to: the ID of the destination where the incoming calls to this number should be forwarded.
        :param connect_to_type: The destination type where the calls should be forwarded. Accepted values 'user', 'phnum', 'sip' and 'app'.
        """

        # Checks
        if not number:
            return util.prepare_error(
                code=const.ERR_NUM_UPDATE_EMPTY,
                message='number is required to update call settings'
            )
        if not connect_to:
            return util.prepare_error(
                code=const.ERR_NUM_UPDATE_EMPTY,
                message='connect_to is required to update call settings'
            )
        if connect_to_type not in const.CONST_CONNECT_TO_TYPES:
            return util.prepare_error(
                code=const.ERR_NUM_UPDATE_EMPTY,
                message=f'invalid connect_to_type value - {connect_to_type}'
            )

        # Prepare request
        body = {
            "connect_to_type": connect_to_type,
            "connect_to": connect_to
        }

        url = f'{self._url}{number}'

        # Return result
        return util.send_api_request(
            token=self._token,
            uri=url,
            method='put',
            body=dumps(body)
        )



"""
# Auth

The Auth class is used to manage handle authentication with the Sonetel system. It can be used to create, refresh and fetch tokens.

It contains the following methods:

* `create_token()` - Create an API access token from the user's Sonetel email address and password.
* `get_access_token()` - Get the access token.
* `get_decoded_token()` - Get the decoded access token.
* `get_refresh_token()` - Get the refresh token.

"""
# Import Packages.
from jwt import decode
import requests
from . import _constants as const
from . import exceptions as e

class Auth:
    """
    Authentication class. Create, refresh and fetch tokens.
    """
    def __init__(self, username: str, password: str):

        self.__username = username
        self.__password = password

        # Get access token from API
        token = self.create_token()
        self._access_token = token['access_token']
        self._refresh_token = token['refresh_token']
        self._decoded_token = decode(
            self._access_token,
            audience='api.sonetel.com',
            options={"verify_signature": False}
        )

    def create_token(self,
                     refresh_token: str = '',
                     grant_type: str = 'password',
                     refresh: str = 'yes',
                     ):
        """
        Create an API access token from the user's Sonetel email address and password.
        Optionally, generate a refresh token. Set the ``grant_type`` to ``refresh_token`` to refresh an
        existing access token.

        **Documentation**: https://docs.sonetel.com/docs/sonetel-documentation/YXBpOjExMzI3NDM3-authentication

        :param refresh: Optional. Flag to return refresh token in the response. Accepted values 'yes' and 'no'. Defaults to 'yes'
        :param grant_type: Optional. The OAuth2 grant type - `password` and `refresh_token` accepted. Defaults to 'password'
        :param refresh_token: Optional. Pass the `refresh_token` generated from a previous request in this field to generate a new access_token.

        :return: dict. The access token and refresh token if the request was processed successfully. If the request failed, the error message is returned.
        """

        # Checks
        if grant_type.strip().lower() not in const.CONST_TYPES_GRANT:
            raise e.AuthException(f'invalid grant: {grant_type}')

        if refresh.strip().lower() not in const.CONST_TYPES_REFRESH:
            refresh = 'yes'

        if grant_type.strip().lower() == 'refresh_token' and not refresh_token:
            refresh_token = self._refresh_token

        # Prepare the request body.
        body = f"grant_type={grant_type}&refresh={refresh}"

        # Add the refresh token to the request body if passed to the function
        if grant_type == 'refresh_token':
            body += f"&refresh_token={refresh_token}"
        else:
            body += f"&username={self.__username}&password={self.__password}"

        # Prepare the request
        auth = (const.CONST_JWT_USER, const.CONST_JWT_PASS)
        headers = {'Content-Type': const.CONTENT_TYPE_AUTH}

        # Send the request
        try:
            req = requests.post(
                url=const.API_URI_AUTH,
                data=body,
                headers=headers,
                auth=auth,
                timeout=60
            )
            req.raise_for_status()
        except requests.exceptions.ConnectionError as err:
            return {'status': 'failed', 'error': 'ConnectionError', 'message': err}
        except requests.exceptions.Timeout:
            return {'status': 'failed', 'error': 'Timeout', 'message': 'Operation timed out. Please try again.'}
        except requests.exceptions.HTTPError as err:
            return {'status': 'failed', 'error': 'Timeout', 'message': err}

        # Check the response and handle accordingly.
        if req.status_code == requests.codes.ok:  # pylint: disable=no-member
            response_json = req.json()

            if refresh_token and grant_type == 'refresh_token':
                self._access_token = response_json["access_token"]
                self._refresh_token = response_json["refresh_token"]
                self._decoded_token = decode(
                    self._access_token,
                    audience='api.sonetel.com',
                    options={"verify_signature": False}
                )

            return response_json
        return {'status': 'failed', 'error': 'Unknown error', 'message': req.text}

    def get_access_token(self):
        """
        Returns the access token.

        Examples:
            >>> from sonetel import Auth
            >>> auth = Auth('username', 'password')
            >>> auth.get_access_token()
            'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJhdWQiOiJhcGkuc29uZXRlbC5jb20iLCJleHAiOjE2MjQwNjY0NzgsImlhdCI6MTYyNDA2Mzg3OCwiaXNzIjoic29uZXRlbC5jb20iLCJqdGkiOiIyMjIyMjIiLCJzdWIiOiJzb25l'

        Args:
            None

        Returns:
            access_token (str): The access token that can be used to access other account resources.
        """
        return self._access_token if hasattr(self, '_access_token') else False

    def get_refresh_token(self):
        """
        Return the refresh token that can be exchanged for a new access token.

        Examples:
            >>> from sonetel import Auth
            >>> auth = Auth('username', 'password')
            >>> auth.get_refresh_token()
            'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJhdWQiOiJhcGkuc29uZXRlbC5jb20iLCJleHAiOjE2MjQwNjY0NzgsImlhdCI6MTYyNDA2Mzg3OCwiaXNzIjoic29uZXRlbC5jb20iLCJqdGkiOiIyMjIyMjIiLCJzdWIiOiJzb25l'

        Args:
            None

        Returns:
            str: The refresh token.
        """
        return self._refresh_token if hasattr(self, '_refresh_token') else False

    def get_decoded_token(self):
        """
        Decodes the access token and returns the decoded token.

        > Note: This method only decodes the payload - it does not verify the signature.

        Examples:
            >>> from sonetel import Auth
            >>> auth = Auth('username', 'password')
            >>> auth.get_decoded_token()
            {'aud': 'api.sonetel.com', 'exp': 1624066478, 'iat': 1624063878, 'iss': 'sonetel.com', 'jti': '222222', 'sub': 'sonetel'}

        Args:
            None

        Returns:
            dict: The decoded token.
        """
        return self._decoded_token if hasattr(self, '_decoded_token') else False



import os
import pytest
from uuid import uuid4
from sonetel import User
from sonetel import Auth
from sonetel import exceptions as e


def access_token():
    try:
        auth = Auth(username=os.getenv('SonetelUsername'), password=os.getenv('SonetelPassword'))
        return auth.get_access_token()
    except e.AuthException as error:
        print(error)
        return None


token = access_token()
if token:
    user = User(access_token=token)
else:
    raise e.AuthException('Cannot get access token')

def test_user_init():
    # Test if the user object is initialized
    assert user is not None

def test_user_failed():
    # Test if the user object initialization fails
    with pytest.raises(Exception) as e_info:
        User(access_token='eyJhbGciOiJIUzI1NiJ9.eyJSb2xlIjoiQWRtaW4iLCJJc3N1ZXIiOiJJc3N1ZXIiLCJ1c2VyX2lkIjoiMjAwMDA0NjAwMCIsInVzZXJfbmFtZSI6InNhbV9kb2VAZXhhbXBsZS5jb20iLCJpc3MiOiJTb25ldGVsTm9kZTEyMyIsImV4cCI6IjE2ODQ0NzIxMDkiLCJpYXQiOiIxNjgxODgwMTA5IiwiYWNjX2lkIjoiMTIzNCIsImNsaWVudF9pZCI6InNvbmV0ZWwtd2ViIn0.ajVzgRfeLwB1IuMzldnzrmoUtUFFmh3QGmWQcs-3Flc')

def test_get_all_users():
    response = user.get(all_users=True)
    print(response['response'])
    assert response['status'] == 'success'

def test_get_invalid_user():
    user_invalid = user.get(userid='1234567890')
    assert user_invalid['status'] == 'success'
    assert user_invalid['response'] == None

def test_get_user():
    # Get a specific user
    response = user.get(userid=os.getenv('SonetelTestUserId'))
    assert response['response']['user_id'] == os.getenv('SonetelTestUserId')
    assert response['status'] == 'success'

def test_get_current_user():
    # Get current user
    response = user.get()
    assert response['status'] == 'success'
    assert response['response']['user_id'] == os.getenv('SonetelSelfUserId')
    assert response['response']['email'] == os.getenv('SonetelUsername')

def test_add_delete_user():

    # Add a new user
    email = f"{str(uuid4())}@example.com"
    password = str(uuid4())[:30]

    response_add = user.add(
        email=email,
        f_name='Test',
        l_name='User',
        user_type='regular',
        password=password
    )
    if response_add['status'] == 'failed':
        raise e.UserException(response_add['response']['detail'])
    assert response_add['status'] == 'success'
    user_to_delete = response_add['response']['user_id']
    assert response_add['response']['email'] == email
    assert response_add['response']['type'] == 'regular'

    # Delete the new user added
    response_del = user.delete(userid=user_to_delete)
    assert response_del['status'] == 'success'
    assert response_del['response'] == 'User deleted successfully'



class SonetelException(Exception):
    """
    General exception class.
    """
    def __init__(self, message):
        super().__init__(message)

class AuthException(SonetelException):
    """
    Authentication errors
    """
    pass

class AccountException(SonetelException):
    """
    Error related to the Sonetel account.
    """

class NumberException(SonetelException):
    """
    Phone number related error
    """
    pass

class VoiceAppException(SonetelException):
    """
    Errors related to voice apps
    """
    pass

class UserException(SonetelException):
    """
    Errors related to the User API
    """
    pass



from setuptools import setup

setup(
    name='sonetel',
    version='0.2.1',
    packages=['sonetel'],
    url='https://github.com/Sonetel/sonetel-python',
    license='MIT',
    author='aashish',
    author_email='dev.support@sonetel.com',
    description='A simple python wrapper for using Sonetel\'s REST APIs'
)



"""
Voice apps
"""
from . import _constants as const
from . import exceptions as e
from . import utilities as util


class VoiceApp(util.Resource):
    """
    Voice app class.
    """

    def __init__(self, access_token):
        if not access_token:
            raise e.AuthException("access_token is required")

        super().__init__(access_token=access_token)
        self._url = f"{const.API_URI_BASE}{const.API_ENDPOINT_ACCOUNT}{self._accountid}{const.API_ENDPOINT_VOICEAPP}"

    def get(self, app_id: str = None):
        """
        Get voice apps. Specify the app_id to get a specific voice app or leave it blank to get all voice apps in the Sonetel account.

        :param app_id: Optional. The ID of the voice app to get. Defaults to None.
        """
        if app_id:
            self._url = f"{self._url}/{app_id}"

        return util.send_api_request(token=self._token, uri=self._url)

    def delete(self, app_id: str):
        """
        Delete a voice app.

        :param app_id: The ID of the voice app to delete.
        """
        self._url = f"{self._url}/{app_id}"

        return util.send_api_request(token=self._token, uri=self._url, method="DELETE")



"""
# Account

The Account class is used to manage your Sonetel account information. You can use it to get information about your account such as the current prepaid balance, the account ID, and more.

You can also update your account information such as the company name, language, and timezone.

The class contains the following methods:

* `get()` - Get information about the Sonetel account.
* `update()` - Update your account information.
* `get_balance()` - Get the current prepaid balance in the account.
* `get_accountid()` - Get your account ID.

"""
from json import dumps
from . import utilities as util
from . import _constants as const
from . import exceptions as e

class Account(util.Resource):
    def __init__(self, access_token: str):
        if not access_token:
            raise e.AuthException('access_token missing')

        super().__init__(access_token=access_token)
        self._url = f'{const.API_URI_BASE}/{const.API_ENDPOINT_ACCOUNT}/{self._accountid}'

    def get(self) -> dict:
        """
        Get information about the Sonetel account.

        Examples:
            >>> account = Account(access_token='your_access_token')
            >>> acc_info = account.get()
            >>> print(acc_info['response']['name'])
            'ACME Inc.'
            >>> print(acc_info['response']['currency'])
            'USD'
        
        Args:
            None

        Returns:
            dict: The account information if the request was processed successfully.
        """

        return util.send_api_request(
            token=self._token,
            uri=self._url,
            method='get',
        )

    def update(self, name: str = '', language: str = '', timezone: str = '') -> dict:
        """
        Update your account information. Pass one or more of the following parameters:

        Examples:
            >>> account = Account(access_token='your_access_token')
            >>> acc_info = account.update(name='ACME Inc.', language='en', timezone='Europe/Stockholm')
            >>> print(acc_info['response']['name'])
            'ACME Inc.'

        Args:
            name (str): New company name.
            language (str): The ID of the new language you want to switch to. Changes the language you see in app.sonetel.com
            timezone (str): The ID of the new timezone you want to switch to.

        Returns:
            dict: The updated account information if the request was processed successfully.

        """

        body = {}
        if name:
            body['name'] = name
        if language:
            body['language'] = language
        if timezone:
            body['timezone_details'] = {
                "zone_id": timezone
            }
        if len(body) == 0:
            return util.prepare_error(
                code=const.ERR_ACCOUNT_UPDATE_BODY_EMPTY,
                message='request body cannot be empty'
            )

        return util.send_api_request(
            token=self._token,
            uri=self._url,
            method='put',
            body=dumps(body)
        )

    def get_balance(self, currency: bool = False) -> str:
        """
        Get the current prepaid balance in the account

        Examples:
            >>> account = Account(access_token='your_access_token')
            >>> print(account.get_balance())
            '1.23'
            >>> print(account.get_balance(currency=True))
            '1.23 USD'
        
        Args:
            currency (bool): Optional. Set to true if currency should be returned in the response.

        Returns:
            str: A string representing the current prepaid balance in the Sonetel account.

        """

        response = util.send_api_request(
            token=self._token,
            uri=self._url,
            method='get',
        )

        balance = response['response']['credit_balance']
        if currency:
            balance += f" {response['response']['currency']}"

        return balance

    def get_accountid(self) -> str:
        """
        Get your account ID. The account ID is used to uniquely identify your Sonetel account.

        Examples:
            >>> account = Account(access_token='your_access_token')
            >>> print(account.get_accountid())
            '123456789'

        Args:
            None

        Returns:
            str: The account ID.
        """
        return self._accountid
